@@ -598,6 +598,51 @@ pub fn using<T, R>(v: T, mut f: impl FnMut(T) -> R) -> R {
     f(v)
 }
 
+/// Create an implicit variable, perform some side effects, and return it
+/// ## Example
+/// ```rust
+/// # use batch_oper::Effect;
+/// let v = Some(1);
+/// let v = v.effect(|v| { assert_eq!(*v, 1) });
+/// assert_eq!(v, Some(1));
+/// ```
+pub trait Effect<T> {
+    /// Create an implicit variable, perform some side effects, and return it
+    /// ## Example
+    /// ```rust
+    /// # use batch_oper::Effect;
+    /// let v = Some(1);
+    /// let v = v.effect(|v| { assert_eq!(*v, 1) });
+    /// assert_eq!(v, Some(1));
+    /// ```
+    fn effect<F: FnOnce(&T)>(self, f: F) -> Self;
+}
+impl<T> Effect<T> for Option<T> {
+    #[inline(always)]
+    fn effect<F: FnOnce(&T)>(self, f: F) -> Self {
+        if let Some(ref v) = self {
+            f(v);
+        }
+        self
+    }
+}
+impl<T, E> Effect<T> for Result<T, E> {
+    #[inline(always)]
+    fn effect<F: FnOnce(&T)>(self, f: F) -> Self {
+        if let Ok(ref v) = self {
+            f(v);
+        }
+        self
+    }
+}
+impl<T, S: std::ops::Deref<Target = T>> Effect<T> for &S {
+    #[inline(always)]
+    fn effect<F: FnOnce(&T)>(self, f: F) -> Self {
+        f(self.deref());
+        self
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -677,11 +722,11 @@ mod tests {
     #[test]
     fn test_fn() {
         let v = 1;
-        let v = effect(v, |v| { assert_eq!(*v, 1) });
+        let v = effect(v, |v| assert_eq!(*v, 1));
         assert_eq!(v, 1);
-        let mut v = using(v, |v| { v + 1 });
+        let mut v = using(v, |v| v + 1);
         assert_eq!(v, 2);
-        using(&mut v, |v| { *v = 3 });
+        using(&mut v, |v| *v = 3);
         assert_eq!(v, 3);
     }
 }
